@@ -1,6 +1,6 @@
 /*
-  Copyright (C) 2006, 2008, 2009 Rocky Bernstein <rocky@gnu.org>
-  
+  Copyright (C) 2006, 2008, 2009, 2021 Rocky Bernstein <rocky@gnu.org>
+
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
@@ -22,14 +22,14 @@
    to use. Otherwise a compiled-in default image name (that
    comes with the libcdio distribution) will be used.
 
-   This program can be compiled with either a C or C++ compiler. In 
+   This program can be compiled with either a C or C++ compiler. In
    the distribution we prefer C++ just to make sure we haven't broken
    things on the C++ side.
  */
 
 /* This is the CD-image with an ISO-9660 filesystem */
 #define ISO9660_IMAGE_PATH "../../../"
-#define ISO9660_IMAGE ISO9660_IMAGE_PATH "test/isofs-m1.cue"
+#define ISO9660_IMAGE ISO9660_IMAGE_PATH "test/data/isofs-m1.cue"
 
 #define ISO9660_PATH "/"
 #define ISO9660_FILENAME "COPYING"
@@ -66,7 +66,7 @@
   delete (p_stat);				\
   delete (p_iso);				\
   return rc;					\
-  
+
 
 int
 main(int argc, const char *argv[])
@@ -79,21 +79,21 @@ main(int argc, const char *argv[])
   char translated_name[256];
   char untranslated_name[256] = ISO9660_PATH;
   ISO9660::FS *p_iso = new ISO9660::FS;
-  
+
   if (argc > 3) {
     printf("usage %s [CD-ROM-or-image [filename]]\n", argv[0]);
     printf("Extracts filename from CD-ROM-or-image.\n");
     return 1;
   }
-  
-  if (argc > 1) 
+
+  if (argc > 1)
     psz_image = argv[1];
-  else 
+  else
     psz_image = ISO9660_IMAGE;
 
-  if (argc > 2) 
+  if (argc > 2)
     psz_fname = argv[2];
-  else 
+  else
     psz_fname = ISO9660_FILENAME;
 
   strcat(untranslated_name, psz_fname);
@@ -105,9 +105,9 @@ main(int argc, const char *argv[])
 
   p_stat = p_iso->stat(psz_fname);
 
-  if (!p_stat) 
+  if (!p_stat)
     {
-      fprintf(stderr, 
+      fprintf(stderr,
 	      "Could not get ISO-9660 file information for file %s\n",
 	      untranslated_name);
       delete(p_iso);
@@ -115,7 +115,7 @@ main(int argc, const char *argv[])
     }
 
   iso9660_name_translate(psz_fname, translated_name);
-  
+
   if (!(p_outfd = fopen (translated_name, "wb")))
     {
       perror ("fopen()");
@@ -131,7 +131,7 @@ main(int argc, const char *argv[])
       {
 	char buf[ISO_BLOCKSIZE];
 	const lsn_t lsn = p_stat->p_stat->lsn + i;
-	
+
 	memset (buf, 0, ISO_BLOCKSIZE);
 
 	try {
@@ -142,9 +142,9 @@ main(int argc, const char *argv[])
 		  (long unsigned int) lsn, e.get_msg());
 	  my_exit(4);
 	}
-	
+
 	fwrite (buf, ISO_BLOCKSIZE, 1, p_outfd);
-	
+
 	if (ferror (p_outfd))
 	  {
 	    perror ("fwrite()");
@@ -152,8 +152,8 @@ main(int argc, const char *argv[])
 	  }
       }
   }
-  
-  
+
+
   fflush (p_outfd);
 
   /* Make sure the file size has the exact same byte size. Without the
@@ -162,7 +162,7 @@ main(int argc, const char *argv[])
   if (ftruncate (fileno (p_outfd), p_stat->p_stat->size))
     perror ("ftruncate()");
 
-  printf("Extraction of file '%s' from '%s' successful.\n", 
+  printf("Extraction of file '%s' from '%s' successful.\n",
 	 translated_name, untranslated_name);
 
   my_exit(0);
