@@ -19,8 +19,11 @@
 /* Simple program to show drivers installed and what the default
    CD-ROM drive is. See also corresponding C program of a similar
    name. */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#define __CDIO_CONFIG_H__ 1
+#endif
 
-#include <cdio/cdio.h>
 #ifdef HAVE_STDIO_H
 #include <stdio.h>
 #endif
@@ -31,6 +34,8 @@
 #include <sys/types.h>
 #endif
 
+#include <cdio/cdio.h>
+
 #define _(x) x
 
 /* Prints out drive capabilities */
