@@ -1,8 +1,7 @@
 /*
-  $Id: isolist.cpp,v 1.2 2008/03/24 15:30:56 karl Exp $
+  Copyright (C) 2004, 2006, 2008, 2009 2017, 2021 Rocky Bernstein
+  <rocky@gnu.org>
 
-  Copyright (C) 2004, 2006, 2008 Rocky Bernstein <rocky@gnu.org>
-  
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
@@ -25,22 +24,23 @@
    use in the listing. Otherwise a compiled-in default ISO 9660 image
    name (that comes with the libcdio distribution) will be used.
 
-   This program can be compiled with either a C or C++ compiler. In 
+   This program can be compiled with either a C or C++ compiler. In
    the distributuion we perfer C++ just to make sure we haven't broken
    things on the C++ side.
  */
 
 /* Set up a CD-DA image to test on which is in the libcdio distribution. */
 #define ISO9660_IMAGE_PATH "../../"
-#define ISO9660_IMAGE ISO9660_IMAGE_PATH "test/copying.iso"
+#define ISO9660_IMAGE ISO9660_IMAGE_PATH "test/data/copying.iso"
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+#include "config.h"
+#define __CDIO_CONFIG_H__ 1
 #endif
-#include <cdio/iso9660.h>
 
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
-
+#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -54,32 +54,34 @@
 #include <sys/types.h>
 #endif
 
+#include <cdio/iso9660.h>
+
 #define print_vd_info(title, fn)	  \
   if (fn(p_iso, &psz_str)) {		  \
     printf(title ": %s\n", psz_str);	  \
   }					  \
   free(psz_str);			  \
-  psz_str = NULL;			  
+  psz_str = NULL;
 
 
 int
 main(int argc, const char *argv[])
 {
-  CdioList_t *p_entlist;
+  CdioISO9660FileList_t *p_entlist;
   CdioListNode_t *p_entnode;
   char const *psz_fname;
   iso9660_t *p_iso;
   const char *psz_path="/";
 
-  if (argc > 1) 
+  if (argc > 1)
     psz_fname = argv[1];
-  else 
+  else
     psz_fname = ISO9660_IMAGE;
 
   p_iso = iso9660_open (psz_fname);
-  
+
   if (NULL == p_iso) {
-    fprintf(stderr, "Sorry, couldn't open %s as an ISO-9660 image\n", 
+    fprintf(stderr, "Sorry, couldn't open %s as an ISO-9660 image\n",
 	    psz_fname);
     return 1;
   }
@@ -94,28 +96,27 @@ main(int argc, const char *argv[])
     print_vd_info("Volume     ", iso9660_ifs_get_volume_id);
     print_vd_info("Volume Set ", iso9660_ifs_get_volumeset_id);
   }
-    
+
   p_entlist = iso9660_ifs_readdir (p_iso, psz_path);
-    
+
   /* Iterate over the list of nodes that iso9660_ifs_readdir gives  */
-  
+
   if (p_entlist) {
     _CDIO_LIST_FOREACH (p_entnode, p_entlist)
     {
       char filename[4096];
-      iso9660_stat_t *p_statbuf = 
+      iso9660_stat_t *p_statbuf =
 	(iso9660_stat_t *) _cdio_list_node_data (p_entnode);
       iso9660_name_translate(p_statbuf->filename, filename);
-      printf ("%s [LSN %6d] %8u %s%s\n", 
+      printf ("%s [LSN %6d] %8u %s%s\n",
 	      2 == p_statbuf->type ? "d" : "-",
 	      p_statbuf->lsn, p_statbuf->size, psz_path, filename);
     }
-    
-    _cdio_list_free (p_entlist, true);
+
+   iso9660_filelist_free(p_entlist);
   }
-  
+
 
   iso9660_close(p_iso);
   return 0;
 }
-
