@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005, 2006 2008 Rocky Bernstein <rocky@gnu.org>
+    Copyright (C) 2005, 2006 2008, 2012 Rocky Bernstein <rocky@gnu.org>
 
     See also rock.c by Eric Youngdale (1993) from GNU/Linux 
     This is Copyright 1993 Yggdrasil Computing, Incorporated
@@ -26,8 +26,8 @@
 */
 
 
-#ifndef __CDIO_ROCK_H__
-#define __CDIO_ROCK_H__
+#ifndef CDIO_ROCK_H_
+#define CDIO_ROCK_H_
 
 #include <cdio/types.h>
 
@@ -35,16 +35,6 @@
 extern "C" {
 #endif /* __cplusplus */
 
-/* MSYS 1.0.10 with MinGW 3.4.2 (and perhaps others) don't have
-   S_ISSOCK() or S_ISLNK() macros, so we'll roll our own. */
-#if !defined(HAVE_S_ISSOCK) && !defined(S_ISSOCK)
-#define S_ISSOCK(st_mode) ((((st_mode)) & 0170000) == (0140000))
-#endif
-
-#if !defined(HAVE_S_ISLNK) && !defined(S_ISLNK)
-#define S_ISLNK(st_mode) ((((st_mode)) & 0170000) == (0010000))
-#endif
-
 /*! An enumeration for some of the ISO_ROCK_* \#defines below. This isn't
   really an enumeration one would really use in a program it is to
   be helpful in debuggers where wants just to refer to the ISO_ROCK_*
@@ -108,24 +98,24 @@ PRAGMA_BEGIN_PACKED
 
 /*! system-use-sharing protocol */
 typedef struct iso_su_sp_s{
-  unsigned char magic[2];
+  uint8_t       magic[2];
   uint8_t       skip;
 } GNUC_PACKED iso_su_sp_t;
 
 /*! system-use extension record */
 typedef struct iso_su_er_s {
   iso711_t      len_id;  /**< Identifier length. Value 10?. */
-  unsigned char len_des;
-  unsigned char len_src;
+  uint8_t       len_des;
+  uint8_t       len_src;
   iso711_t      ext_ver; /**< Extension version. Value 1? */
-  char data[EMPTY_ARRAY_SIZE];
+  char          data[EMPTY_ARRAY_SIZE];
 } GNUC_PACKED iso_su_er_t;
 
 typedef struct iso_su_ce_s {
-  char extent[8];
-  char offset[8];
-  char size[8];
-} iso_su_ce_t;
+  iso733_t      extent;
+  iso733_t      offset;
+  iso733_t      size;
+} GNUC_PACKED iso_su_ce_t;
 
 /*! POSIX file attributes, PX. See Rock Ridge Section 4.1.2 */
 typedef struct iso_rock_px_s {
@@ -162,10 +152,10 @@ typedef enum {
   ISO_ROCK_SL_ROOT     = 8
 } iso_rock_sl_flag_t;
 
-#define	ISO_ROCK_SL_CONTINUE 1
-#define	ISO_ROCK_SL_CURRENT  2
-#define	ISO_ROCK_SL_PARENT   4
-#define	ISO_ROCK_SL_ROOT     8
+#define ISO_ROCK_SL_CONTINUE 1
+#define ISO_ROCK_SL_CURRENT  2
+#define ISO_ROCK_SL_PARENT   4
+#define ISO_ROCK_SL_ROOT     8
 
 typedef struct iso_rock_sl_part_s {
   uint8_t flags;
@@ -175,7 +165,7 @@ typedef struct iso_rock_sl_part_s {
 
 /*! Symbolic link. See Rock Ridge Section 4.1.3 */
 typedef struct iso_rock_sl_s {
-  unsigned char flags;
+  uint8_t flags;
   iso_rock_sl_part_t link;
 } GNUC_PACKED iso_rock_sl_t ;
 
@@ -188,24 +178,24 @@ typedef enum {
   ISO_ROCK_NM_PARENT   = 4,
 } iso_rock_nm_flag_t;
 
-#define	ISO_ROCK_NM_CONTINUE 1
-#define	ISO_ROCK_NM_CURRENT  2
-#define	ISO_ROCK_NM_PARENT   4
+#define ISO_ROCK_NM_CONTINUE 1
+#define ISO_ROCK_NM_CURRENT  2
+#define ISO_ROCK_NM_PARENT   4
 
 
 typedef struct iso_rock_nm_s {
-  unsigned char flags;
+  uint8_t flags;
   char name[EMPTY_ARRAY_SIZE];
 } GNUC_PACKED iso_rock_nm_t ;
 
 /*! Child link. See Section 4.1.5.1 */
 typedef struct iso_rock_cl_s {
-  char location[1];
+  iso733_t location;
 } GNUC_PACKED iso_rock_cl_t ;
 
 /*! Parent link. See Section 4.1.5.2 */
 typedef struct iso_rock_pl_s {
-  char location[1];
+  iso733_t location;
 } GNUC_PACKED iso_rock_pl_t ;
 
 /*! These are the bits and their meanings for flags in the TF structure. */
@@ -274,9 +264,29 @@ typedef struct iso_extension_record_s {
     iso_rock_cl_t  CL;  /**< Rock Ridge child link */
     iso_rock_pl_t  PL;  /**< Rock Ridge parent link */
     iso_rock_tf_t  TF;  /**< Rock Ridge timestamp(s) for a file */
+    iso_rock_sf_t  SF;  /**< Rock Ridge sparse file */
   } u;
 } GNUC_PACKED iso_extension_record_t;
 
+/* Bits for the u_su_fields of iso_rock_statbuf_t */
+#define ISO_ROCK_SUF_SP 0x00000001
+#define ISO_ROCK_SUF_ER 0x00000002
+#define ISO_ROCK_SUF_CE 0x00000004
+#define ISO_ROCK_SUF_PX 0x00000008
+#define ISO_ROCK_SUF_PN 0x00000010
+#define ISO_ROCK_SUF_SL 0x00000020
+#define ISO_ROCK_SUF_NM 0x00000040
+#define ISO_ROCK_SUF_TF 0x00000080
+#define ISO_ROCK_SUF_CL 0x00000100
+#define ISO_ROCK_SUF_PL 0x00000200
+#define ISO_ROCK_SUF_RE 0x00000400
+#define ISO_ROCK_SUF_SF 0x00000800
+
+#define ISO_ROCK_SUF_FORMAL (ISO_ROCK_SUF_ER | ISO_ROCK_SUF_PX | ISO_ROCK_SUF_PN | \
+                             ISO_ROCK_SUF_SL | ISO_ROCK_SUF_NM | ISO_ROCK_SUF_CL | \
+                             ISO_ROCK_SUF_PL | ISO_ROCK_SUF_RE | ISO_ROCK_SUF_TF | \
+                             ISO_ROCK_SUF_SF)
+
 typedef struct iso_rock_time_s {
   bool          b_used;     /**< If true, field has been set and  is valid. 
                                Otherwise remaning fields are meaningless. */
@@ -316,19 +326,23 @@ typedef struct iso_rock_statbuf_s {
                                          9660:9.5.6. */
   iso_rock_time_t effective;          /**< Effective time; See ISO 9660:9.5.7.
                                        */
-  uint32_t i_rdev;                    /**< the upper 16-bits is major device 
+  uint32_t        i_rdev;             /**< the upper 16-bits is major device 
                                          number, the lower 16-bits is the
                                          minor device number */
+  uint32_t        u_su_fields;        /**< System Use field attributes */
 
 } iso_rock_statbuf_t;
-  
+
 PRAGMA_END_PACKED
 
 /*! return length of name field; 0: not found, -1: to be ignored */
-int get_rock_ridge_filename(iso9660_dir_t * de, /*out*/ char * retname, 
+int get_rock_ridge_filename(iso9660_dir_t * de,
+                            /*in*/ void * p_iso,
+                            /*out*/ char * retname,
                             /*out*/ iso9660_stat_t *p_stat);
 
-  int parse_rock_ridge_stat(iso9660_dir_t *de, /*out*/ iso9660_stat_t *p_stat);
+int parse_rock_ridge_stat(iso9660_dir_t *de,
+                          /*out*/ iso9660_stat_t *p_stat);
 
   /*!
     Returns POSIX mode bitstring for a given file.
@@ -383,7 +397,7 @@ extern iso_rock_tf_flag_t iso_rock_tf_flag;
 }
 #endif /* __cplusplus */
 
-#endif /* __ISO_ROCK_H__ */
+#endif /* CDIO_ROCK_H_ */
 
 /* 
  * Local variables:
