@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003, 2004, 2005, 2006, 2008, 2010, 2011, 2012, 2014
+  Copyright (C) 2003-2006, 2008, 2010-2012, 2014, 2017
   Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
@@ -73,6 +73,7 @@
 
 #if defined (_MSC_VER) || defined (_XBOX)
 #undef IN
+extern const char* is_cdrom_aspi(const char drive_letter);
 #else
 #include "aspi32.h"
 #endif
@@ -209,7 +210,7 @@ _cdio_mciSendCommand(int id, UINT msg, DWORD flags, void *arg)
   if ( mci_error ) {
     char error[256];
 
-    mciGetErrorString(mci_error, error, 256);
+    mciGetErrorStringA(mci_error, error, 256);
     cdio_warn("mciSendCommand() error: %s", error);
   }
   return(mci_error == 0);
@@ -382,7 +383,7 @@ free_win32 (void *p_user_data)
    Reads an audio device into data starting from lsn.
    Returns 0 if no error.
  */
-static int
+static driver_return_code_t
 read_audio_sectors (void *p_user_data, void *p_buf, lsn_t i_lsn,
 		    unsigned int i_blocks)
 {
@@ -708,6 +709,36 @@ get_arg_win32 (void *p_user_data, const char key[])
   return NULL;
 }
 
+/**
+  Read CD-Text binary data.
+*/
+static uint8_t *
+read_cdtext_win32(void *p_user_data)
+{
+  const _img_private_t *p_env = p_user_data;
+
+  if( p_env->hASPI ) {
+    return read_cdtext_generic(p_user_data);
+  } else {
+    return read_cdtext_win32ioctl(p_user_data);
+  }
+}
+
+/**
+  Read CD-Text and return cdtext_t structure.
+*/
+static cdtext_t *
+get_cdtext_win32 (void *p_user_data)
+{
+  const _img_private_t *p_env = p_user_data;
+
+  if( p_env->hASPI ) {
+    return get_cdtext_generic(p_user_data);
+  } else {
+    return get_cdtext_win32ioctl(p_user_data);
+  }
+}
+
 /*!
   Return the media catalog number MCN.
 
@@ -818,12 +849,15 @@ _cdio_get_track_msf(void *p_user_data, track_t i_tracks, msf_t *p_msf)
   if (!p_env->gen.toc_init)
     if (!read_toc_win32 (p_env)) return false;
 
-  if (i_tracks == CDIO_CDROM_LEADOUT_TRACK) i_tracks = p_env->gen.i_tracks+1;
+  if (i_tracks == CDIO_CDROM_LEADOUT_TRACK)
+    i_tracks = p_env->gen.i_tracks + p_env->gen.i_first_track;
 
-  if (i_tracks > p_env->gen.i_tracks+1 || i_tracks == 0) {
+  if (i_tracks > (p_env->gen.i_tracks + p_env->gen.i_first_track)
+      || i_tracks < p_env->gen.i_first_track) {
     return false;
   } else {
-    cdio_lsn_to_msf(p_env->tocent[i_tracks-1].start_lsn, p_msf);
+    cdio_lsn_to_msf(
+          p_env->tocent[i_tracks - p_env->gen.i_first_track].start_lsn, p_msf);
     return true;
   }
 }
@@ -994,8 +1028,8 @@ cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
   _funcs.eject_media            = eject_media_win32;
   _funcs.free                   = free_win32;
   _funcs.get_arg                = get_arg_win32;
-  _funcs.get_cdtext             = get_cdtext_generic;
-  _funcs.get_cdtext_raw         = read_cdtext_generic;
+  _funcs.get_cdtext             = get_cdtext_win32;
+  _funcs.get_cdtext_raw         = read_cdtext_win32;
   _funcs.get_default_device     = cdio_get_default_device_win32;
   _funcs.get_devices            = cdio_get_devices_win32;
   _funcs.get_disc_last_lsn      = get_disc_last_lsn_win32;
@@ -1030,13 +1064,18 @@ cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
   _funcs.set_speed              = set_drive_speed_mmc;
 
   _data                 = calloc(1, sizeof (_img_private_t));
+  if (NULL == _data) {
+    goto error_exit;
+  }
   _data->access_mode    = str_to_access_mode_win32(psz_access_mode);
   _data->gen.init       = false;
   _data->gen.fd         = -1;
 
   if (NULL == psz_orig_source) {
     psz_source=cdio_get_default_device_win32();
-    if (NULL == psz_source) return NULL;
+    if (NULL == psz_source) {
+      goto error_exit;
+    }
     set_arg_win32(_data, "source", psz_source);
     free(psz_source);
   } else {
@@ -1046,21 +1085,23 @@ cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
       /* The below would be okay as an info message if all device
 	 drivers worked this way. */
       cdio_debug ("source %s is a not a device", psz_orig_source);
-      free(_data);
-      return NULL;
+      goto error_exit;
     }
   }
 
   ret = cdio_new ((void *)_data, &_funcs);
-  if (ret == NULL) return NULL;
-
-  if (init_win32(_data))
-    return ret;
-  else {
-    free_win32 (_data);
-    return NULL;
+  if (ret == NULL) {
+    goto error_exit;
   }
 
+  if (init_win32(_data)) {
+    return ret;
+  }
+  
+
+ error_exit:
+  free_win32(_data);
+  return(NULL);
 #else
   return NULL;
 #endif /* HAVE_WIN32_CDROM */
