@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
-  Copyright (C) 2002-2006, 2008-2013, 2017 Rocky Bernstein
+  Copyright (C) 2002-2006, 2008-2013, 2017, 2022 Rocky Bernstein
   <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
@@ -52,6 +52,9 @@
 # else
 #  error "You need a kernel greater than 2.2.16 to have CDROM support"
 # endif
+# if LINUX_VERSION_CODE >= KERNEL_VERSION(5,16,0)
+#   define __CDIO_LINUXCD_USE_TIMED_MEDIA_CHANGED
+# endif
 #else
 #  error "You need <linux/version.h> to have CDROM support"
 #endif
@@ -99,6 +102,16 @@ typedef struct {
 
   struct cdrom_tochdr    tochdr;
 
+#if defined(__CDIO_LINUXCD_USE_TIMED_MEDIA_CHANGED)
+  /* The new TIMED_MEDIA_CHANGED ioctl requires us
+    to store the timestamp of our last check. Since
+    cdio_media_changed takes user data through a constant
+    pointer, we cannot use a direct field, as that would
+    not be writable in that method. Therefore, make this
+    a pointer. */
+  __s64 *last_changed_timestamp;
+#endif
+
 } _img_private_t;
 
 /* Some ioctl() errno values which occur when the tray is empty */
@@ -168,8 +181,8 @@ check_mounts_linux(const char *mtab)
         continue;
       }
 
-      strncpy(mnt_type, mntent->mnt_type, i_mnt_type);
-      strncpy(mnt_dev, mntent->mnt_fsname, i_mnt_dev);
+      strcpy(mnt_type, mntent->mnt_type);
+      strcpy(mnt_dev, mntent->mnt_fsname);
 
       /* Handle "supermount" filesystem mounts */
       if ( strcmp(mnt_type, "supermount") == 0 ) {
@@ -360,6 +373,23 @@ audio_stop_linux (void *p_user_data)
   return ioctl(p_env->gen.fd, CDROMSTOP);
 }
 
+/*!
+  Release and free resources associated with cd for linux driver.
+ */
+static void
+free_linux (void *p_user_data)
+{
+#ifdef __CDIO_LINUXCD_USE_TIMED_MEDIA_CHANGED
+  _img_private_t *p_env = p_user_data;
+  if (p_env->last_changed_timestamp) {
+    free(p_env->last_changed_timestamp);
+    p_env->last_changed_timestamp = 0;
+  }
+#endif
+
+  cdio_generic_free(p_user_data);
+}
+
 static bool
 is_mmc_supported(void *user_data)
 {
@@ -505,7 +535,19 @@ get_last_session_linux (void *p_user_data,
 static int
 get_media_changed_linux (const void *p_user_data) {
   const _img_private_t *p_env = p_user_data;
+#if defined(__CDIO_LINUXCD_USE_TIMED_MEDIA_CHANGED)
+  struct cdrom_timed_media_change_info info = {
+    .last_media_change  = *(p_env->last_changed_timestamp),
+    .media_flags        = 0
+  };
+  if (ioctl(p_env->gen.fd, CDROM_TIMED_MEDIA_CHANGE, &info)) {
+    return DRIVER_OP_ERROR;
+  }
+  *(p_env->last_changed_timestamp) = info.last_media_change;
+  return info.media_flags & MEDIA_CHANGED_FLAG;
+#else
   return ioctl(p_env->gen.fd, CDROM_MEDIA_CHANGED, 0);
+#endif
 }
 
 /*!
@@ -647,13 +689,13 @@ static int is_mounted (const char * device, char * target) {
   char real_device_1[PATH_MAX];
   char real_device_2[PATH_MAX];
 
-  char file_device[PATH_MAX];
-  char file_target[PATH_MAX];
+  struct mntent *fs;
+
+  fp = setmntent("/proc/mounts", "r");
 
-  fp = fopen ( "/proc/mounts", "r");
   /* Older systems just have /etc/mtab */
   if(!fp)
-    fp = fopen ( "/etc/mtab", "r");
+    fp = setmntent("/etc/mtab", "r");
 
   /* Neither /proc/mounts nor /etc/mtab could be opened, give up here */
   if(!fp) return 0;
@@ -666,19 +708,19 @@ static int is_mounted (const char * device, char * target) {
 
   /* Read entries */
 
-  while ( fscanf(fp, "%s %s %*s %*s %*d %*d\n", file_device, file_target) != EOF ) {
-      if (NULL == cdio_realpath(file_device, real_device_2)) {
+  while ((fs = getmntent(fp)) != NULL) {
+      if (NULL == cdio_realpath(fs->mnt_fsname, real_device_2)) {
           cdio_debug("Problems resolving device %s: %s\n",
-                     file_device, strerror(errno));
+                     fs->mnt_fsname, strerror(errno));
       }
     if(!strcmp(real_device_1, real_device_2)) {
-      strcpy(target, file_target);
-      fclose(fp);
+      strcpy(target, fs->mnt_dir);
+      endmntent(fp);
       return 1;
     }
 
   }
-  fclose(fp);
+  endmntent(fp);
   return 0;
 }
 
@@ -999,66 +1041,7 @@ static driver_return_code_t
 _read_mode1_sector_linux (void *p_user_data, void *p_data, lsn_t lsn,
                           bool b_form2)
 {
-
-#if 0
-  char buf[M2RAW_SECTOR_SIZE] = { 0, };
-  struct cdrom_msf *p_msf = (struct cdrom_msf *) &buf;
-  msf_t _msf;
-
-  _img_private_t *p_env = p_user_data;
-
-  cdio_lsn_to_msf (lsn, &_msf);
-  p_msf->cdmsf_min0 = cdio_from_bcd8(_msf.m);
-  p_msf->cdmsf_sec0 = cdio_from_bcd8(_msf.s);
-  p_msf->cdmsf_frame0 = cdio_from_bcd8(_msf.f);
-
- retry:
-  switch (p_env->access_mode)
-    {
-    case _AM_NONE:
-      cdio_warn ("no way to read mode1");
-      return 1;
-      break;
-
-    case _AM_IOCTL:
-      if (ioctl (p_env->gen.fd, CDROMREADMODE1, &buf) == -1)
-        {
-          perror ("ioctl()");
-          return 1;
-          /* exit (EXIT_FAILURE); */
-        }
-      break;
-
-    case _AM_READ_CD:
-    case _AM_READ_10:
-      if (_read_mode2_sectors (p_env, buf, lsn, 1,
-                               (p_env->access_mode == _AM_READ_10)))
-        {
-          perror ("ioctl()");
-          if (p_env->access_mode == _AM_READ_CD)
-            {
-              cdio_info ("READ_CD failed; switching to READ_10 mode...");
-              p_env->access_mode = _AM_READ_10;
-              goto retry;
-            }
-          else
-            {
-              cdio_info ("READ_10 failed; switching to ioctl(CDROMREADMODE2) mode...");
-              p_env->access_mode = _AM_IOCTL;
-              goto retry;
-            }
-          return 1;
-        }
-      break;
-    }
-
-  memcpy (p_data, buf + CDIO_CD_SYNC_SIZE + CDIO_CD_HEADER_SIZE,
-          b_form2 ? M2RAW_SECTOR_SIZE: CDIO_CD_FRAMESIZE);
-
-#else
   return cdio_generic_read_form1_sector(p_user_data, p_data, lsn);
-#endif
-  return 0;
 }
 
 /*!
@@ -1185,7 +1168,7 @@ static bool
 read_toc_linux (void *p_user_data)
 {
   _img_private_t *p_env = p_user_data;
-  int i;
+  int i, i_last_track;
   unsigned int u_tracks;
 
   /* read TOC header */
@@ -1196,9 +1179,10 @@ read_toc_linux (void *p_user_data)
   }
 
   p_env->gen.i_first_track = p_env->tochdr.cdth_trk0;
-  p_env->gen.i_tracks      = p_env->tochdr.cdth_trk1;
+  i_last_track             = p_env->tochdr.cdth_trk1;
+  p_env->gen.i_tracks      = i_last_track - p_env->gen.i_first_track + 1;
 
-  u_tracks = p_env->gen.i_tracks - p_env->gen.i_first_track + 1;
+  u_tracks = p_env->gen.i_tracks;
 
   if (u_tracks > CDIO_CD_MAX_TRACKS) {
      cdio_log(CDIO_LOG_WARN, "Number of tracks exceeds maximum (%d vs. %d)\n",
@@ -1208,7 +1192,7 @@ read_toc_linux (void *p_user_data)
 
 
   /* read individual tracks */
-  for (i= p_env->gen.i_first_track; i<=p_env->gen.i_tracks; i++) {
+  for (i= p_env->gen.i_first_track; i <= i_last_track; i++) {
     struct cdrom_tocentry *p_toc =
       &(p_env->tocent[i-p_env->gen.i_first_track]);
 
@@ -1712,7 +1696,7 @@ cdio_open_am_linux (const char *psz_orig_source, const char *access_mode)
     .audio_set_volume      = audio_set_volume_linux,
     .audio_stop            = audio_stop_linux,
     .eject_media           = eject_media_linux,
-    .free                  = cdio_generic_free,
+    .free                  = free_linux,
     .get_arg               = get_arg_linux,
     .get_blocksize         = get_blocksize_mmc,
     .get_cdtext            = get_cdtext_generic,
@@ -1770,6 +1754,10 @@ cdio_open_am_linux (const char *psz_orig_source, const char *access_mode)
   _data->gen.toc_init   = false;
   _data->gen.fd         = -1;
   _data->gen.b_cdtext_error = false;
+#ifdef __CDIO_LINUXCD_USE_TIMED_MEDIA_CHANGED
+  _data->last_changed_timestamp = calloc(1, sizeof (__s64));
+  *(_data->last_changed_timestamp) = 0;
+#endif
 
   if (NULL == psz_orig_source) {
     psz_source=cdio_get_default_device_linux();
@@ -1812,7 +1800,7 @@ cdio_open_am_linux (const char *psz_orig_source, const char *access_mode)
   free(ret);
 
  err_exit:
-    cdio_generic_free(_data);
+    free_linux(_data);
     return NULL;
 
 #else
