@@ -212,7 +212,7 @@ _read_bincue (void *p_user_data, void *data, size_t size)
     final_size += this_size;
     memcpy (p, buf, this_size);
     p += this_size;
-    this_size = cdio_stream_read(p_env->gen.data_source, buf, rem, 1);
+    cdio_stream_read(p_env->gen.data_source, buf, rem, 1);
 
     /* Skip over stuff at end of this sector and the beginning of the next.
      */
@@ -357,17 +357,28 @@ parse_cuefile (_img_private_t *cd, const char *psz_cue_name)
   } else if (0 == strcmp ("CDTEXTFILE", psz_keyword)) {
     if(NULL != (psz_field = strtok (NULL, "\"\t\n\r"))) {
       if (cd) {
-        uint8_t cdt_data[CDTEXT_LEN_BINARY_MAX+4];
-        int size;
+        uint8_t *cdt_data = NULL, *cdt_packs;
+        int size, mmc_len;
         CdioDataSource_t *source;
         char *dirname = cdio_dirname(psz_cue_name);
         char *psz_filename = cdio_abspath(dirname, psz_field);
 
+        cdt_data = calloc(CDTEXT_LEN_BINARY_MAX + 4, 1);
+        if (NULL == cdt_data) {
+          cdio_log(log_level,
+                  "%s line %d: cannot allocate memory for CD-TEXT data buffer",
+                  psz_cue_name, i_line);
+          free(psz_filename);
+          free(dirname);
+          goto err_exit;
+        }
+
         if(NULL == (source = cdio_stdio_new(psz_filename))) {
           cdio_log(log_level, "%s line %d: can't open file `%s' for reading",
 		   psz_cue_name, i_line, psz_field);
           free(psz_filename);
           free(dirname);
+          free(cdt_data);
           goto err_exit;
         }
         size = cdio_stream_read(source, cdt_data, CDTEXT_LEN_BINARY_MAX, 1);
@@ -378,13 +389,27 @@ parse_cuefile (_img_private_t *cd, const char *psz_cue_name)
                    psz_cue_name, i_line, psz_filename);
           free(psz_filename);
           free(dirname);
-	  free(source);
+          free(cdt_data);
+          cdio_stdio_destroy (source);
           goto err_exit;
         }
 
-        /* Truncate header when it is too large. */
-        if (cdt_data[0] > 0x80) {
-          size -= 4;
+        /* Check whether obviously a MMC header is prepended and if so,skip it.
+           cdtext_data_init() wants to see only the text pack bytes.
+        */
+        cdt_packs = cdt_data;
+        if (cdt_data[0] < 0x80) {
+          /* This cannot be a text pack start */
+          mmc_len = CDIO_MMC_GET_LEN16(cdt_data) + 2;
+          if ((size == mmc_len || size == mmc_len + 1) && mmc_len % 18 == 4 &&
+              cdt_data[4] >= 0x80 && cdt_data[4] <= 0x8f) {
+            /* It looks much like a MMC header followed by a text pack start */
+            size -= 4;
+            cdt_packs = cdt_data + 4;
+            cdio_log (CDIO_LOG_INFO,
+                      "%s line %d: skipped 4 bytes of apparent MMC header in CD-TEXT file `%s'\n",
+                      psz_cue_name, i_line, psz_filename);
+          }
         }
 
         /* ignore trailing 0 */
@@ -396,12 +421,13 @@ parse_cuefile (_img_private_t *cd, const char *psz_cue_name)
           cd->gen.cdtext = cdtext_init ();
         }
 
-        if(0 != cdtext_data_init(cd->gen.cdtext, cdt_data, size))
+        if(0 != cdtext_data_init(cd->gen.cdtext, cdt_packs, size))
           cdio_log (log_level, "%s line %d: failed to parse CD-TEXT file `%s'", psz_cue_name, i_line, psz_filename);
 
         cdio_stdio_destroy (source);
         free(psz_filename);
         free(dirname);
+        free(cdt_data);
       }
     } else {
       goto format_error;
@@ -432,6 +458,20 @@ parse_cuefile (_img_private_t *cd, const char *psz_cue_name)
                      "Expecting a track number, got %s", psz_field);
             goto err_exit;
           }
+          if (i_track < 1 || i_track > 99) {
+            cdio_log(log_level,
+                     "Track number out of range 1 to 99, got %s", psz_field);
+            goto err_exit;
+          }
+          if(cd) {
+            if (-1 == i) {
+              cd->gen.i_first_track = i_track;
+            } else if(cd->gen.i_first_track + i + 1 != i_track) {
+              cdio_log(log_level,
+                       "Track number out of sequence. Expected %d, got %d",
+                       cd->gen.i_first_track + i + 1, i_track);
+            }
+          }
         }
         if (NULL != (psz_field = strtok(NULL, " \t\n\r"))) {
           track_info_t  *this_track=NULL;
@@ -748,7 +788,7 @@ parse_cuefile (_img_private_t *cd, const char *psz_cue_name)
               cd->tocent[i].indexes[cd->tocent[i].nindex++] = lba;
 #else
               track_info_t  *this_track=
-                &(cd->tocent[cd->gen.i_tracks - cd->gen.i_first_track]);
+                &(cd->tocent[cd->gen.i_tracks - 1]);
 
               switch (start_index) {
 
@@ -814,6 +854,8 @@ parse_cuefile (_img_private_t *cd, const char *psz_cue_name)
         if (cd) {
           if (NULL == cd->gen.cdtext) {
             cd->gen.cdtext = cdtext_init ();
+            if (NULL == cd->gen.cdtext)
+              goto err_exit;
             cd->gen.cdtext->block[cd->gen.cdtext->block_i].language_code = CDTEXT_LANGUAGE_ENGLISH;
           }
           cdtext_set (cd->gen.cdtext, cdtext_key, (uint8_t*) strtok(NULL, "\"\t\n\r"),
@@ -989,7 +1031,7 @@ _read_mode2_sectors_bincue (void *p_user_data, void *data, lsn_t lsn,
 }
 
 #if !defined(HAVE_GLOB_H) && defined(_WIN32)
-static inline void Win32Glob(const char* pattern, const char* szCurPath, char ***drives, unsigned int *num_files)
+static void Win32Glob(const char* pattern, const char* szCurPath, char ***drives, unsigned int *num_files)
 {
   char szPath[MAX_PATH];
   WIN32_FIND_DATAA ffd;
@@ -1090,7 +1132,8 @@ _get_track_format_bincue(void *p_user_data, track_t i_track)
 
   if (!p_env->gen.init) return TRACK_FORMAT_ERROR;
 
-  if (i_track > p_env->gen.i_tracks || i_track == 0)
+  if (i_track > p_env->gen.i_first_track + p_env->gen.i_tracks - 1
+      || i_track < p_env->gen.i_first_track)
     return TRACK_FORMAT_ERROR;
 
   return p_env->tocent[i_track-p_env->gen.i_first_track].track_format;
@@ -1129,10 +1172,11 @@ _get_lba_track_bincue(void *p_user_data, track_t i_track)
 {
   _img_private_t *p_env = p_user_data;
 
-  if (i_track == CDIO_CDROM_LEADOUT_TRACK) i_track = p_env->gen.i_tracks+1;
+  if (i_track == CDIO_CDROM_LEADOUT_TRACK)
+    i_track = p_env->gen.i_tracks + p_env->gen.i_first_track;
 
   if (i_track <= p_env->gen.i_tracks + p_env->gen.i_first_track
-      && i_track != 0) {
+      && i_track >= p_env->gen.i_first_track) {
     return p_env->tocent[i_track-p_env->gen.i_first_track].start_lba;
   } else
     return CDIO_INVALID_LBA;
@@ -1294,6 +1338,7 @@ cdio_open_cue (const char *psz_cue_name)
   if (NULL == psz_cue_name) return NULL;
 
   p_data                 = calloc(1, sizeof (_img_private_t));
+  if (NULL == p_data) return NULL;
   p_data->gen.init       = false;
   p_data->psz_cue_name   = NULL;
 
