@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003-2006, 2008-2009, 2011-2012, 2014
+  Copyright (C) 2003-2006, 2008-2009, 2011-2012, 2014, 2017
   Rocky Bernstein <rocky@gnu.org>
   Copyright (C) 2001, 2003 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -88,6 +88,8 @@ _register_mapping (_img_private_t *env, lsn_t start_lsn, uint32_t sec_count,
   track_info_t  *this_track=&(env->tocent[env->gen.i_tracks]);
   _mapping_t *_map = calloc(1, sizeof (_mapping_t));
 
+  if (_map == NULL)
+    return;
   _map->start_lsn  = start_lsn;
   _map->sec_count  = sec_count;
   _map->img_offset = img_offset;
@@ -247,16 +249,15 @@ parse_nrg (_img_private_t *p_env, const char *psz_nrg_name,
 	  entries /= sizeof (_cuex_array_t);
 
 	  if (CUES_ID == opcode) {
-	    lsn_t lsn = UINT32_FROM_BE (_entries[0].lsn);
 	    unsigned int idx;
 	    unsigned int i = 0;
-
-	    cdio_debug ("CUES type image detected" );
-
+	    lsn_t lsn; /* = UINT32_FROM_BE (_entries[0].lsn); */
 	    /* CUES LSN has 150 pregap include at beginning? -/
 	       cdio_assert (lsn == 0?);
 	    */
 
+	    cdio_debug ("CUES type image detected" );
+
 	    p_env->is_cues           = true; /* HACK alert. */
 	    p_env->gen.i_tracks      = 0;
 	    p_env->gen.i_first_track = 1;
@@ -385,6 +386,8 @@ parse_nrg (_img_private_t *p_env, const char *psz_nrg_name,
 
 	  /* We include an extra 0 byte so these can be used as C strings.*/
 	  p_env->psz_mcn = calloc(1, CDIO_MCN_SIZE+1);
+	  if (p_env->psz_mcn == NULL)
+	    return false;
 	  memcpy(p_env->psz_mcn, &(_dao_common->psz_mcn), CDIO_MCN_SIZE);
 	  p_env->psz_mcn[CDIO_MCN_SIZE] = '\0';
 
@@ -1107,7 +1110,7 @@ _free_nrg (void *p_user_data)
 
   if (NULL == p_env) return;
   if (NULL != p_env->mapping)
-    _cdio_list_free (p_env->mapping, true);
+    _cdio_list_free (p_env->mapping, true, free);
 
   /* The remaining part of the image is like the other image drivers,
      so free that in the same way. */
@@ -1126,7 +1129,7 @@ _eject_media_nrg(void *obj)
 }
 
 #if !defined(HAVE_GLOB_H) && defined(_WIN32)
-static inline void Win32Glob(const char* pattern, const char* szCurPath, char ***drives, unsigned int *num_files)
+static void Win32Glob(const char* pattern, const char* szCurPath, char ***drives, unsigned int *num_files)
 {
   char szPath[MAX_PATH];
   WIN32_FIND_DATAA ffd;
@@ -1271,11 +1274,15 @@ cdio_is_nrg(const char *psz_nrg)
   _img_private_t *p_env  = calloc(1, sizeof (_img_private_t));
   bool is_nrg = false;
 
-  if (psz_nrg == NULL) return false;
+  if (p_env == NULL || psz_nrg == NULL) {
+    is_nrg = false;
+    goto exit;
+  }
 
   if (!(p_env->gen.data_source = cdio_stdio_new (psz_nrg))) {
     cdio_warn ("can't open nrg image file %s for reading", psz_nrg);
-    return false;
+    is_nrg = false;
+    goto exit;
   }
 
   if (parse_nrg(p_env, psz_nrg, CDIO_LOG_INFO)) {
@@ -1284,11 +1291,15 @@ cdio_is_nrg(const char *psz_nrg)
     size_t psz_len;
     psz_len = strlen(psz_nrg);
     /* At least 4 characters needed for .nrg extension */
-    if ( psz_len < 4 ) return false;
+    if ( psz_len < 4 ) {
+      is_nrg = false;
+      goto exit;
+    }
 
     is_nrg = strncasecmp( psz_nrg+(psz_len-3), "nrg", 3 ) == 0;
 #endif
   }
+ exit:
   _free_nrg(p_env);
   return is_nrg;
 }
@@ -1354,6 +1365,9 @@ cdio_open_nrg (const char *psz_source)
   _funcs.set_arg               = _set_arg_image;
 
   _data                   = calloc(1, sizeof (_img_private_t));
+  if (_data == NULL) {
+    return NULL;
+  }
   _data->gen.init         = false;
 
   _data->gen.i_tracks     = 0;
