@@ -1,5 +1,8 @@
 /*
-  Copyright (C) 2003-2008, 2011-2015, 2017 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2003-2008, 2011-2015, 2017, 2024
+  Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2018, 2020 Pete Batard <pete@akeo.ie>
+  Copyright (C) 2018 Thomas Schmitt <scdbackup@gmx.net>
   Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
@@ -59,6 +62,7 @@
 
 /** Implementation of iso9660_t type */
 struct _iso9660_s {
+  cdio_header_t header;     /**< Internal header - MUST come first. */
   CdioDataSource_t *stream; /**< Stream pointer */
   bool_3way_t b_xa;         /**< true if has XA attributes. */
   bool_3way_t b_mode2;      /**< true if has mode 2, false for mode 1. */
@@ -90,8 +94,7 @@ struct _iso9660_s {
 			         filesystem inside that it may be
 			         different.
 			     */
-    bool b_have_superblock; /**< Superblock has been read in? */
-
+  bool b_have_superblock;   /**< Superblock has been read in? */
 };
 
 static long int iso9660_seek_read_framesize (const iso9660_t *p_iso,
@@ -172,8 +175,10 @@ iso9660_open_ext_private (const char *psz_path,
 {
   iso9660_t *p_iso = (iso9660_t *) calloc(1, sizeof(iso9660_t)) ;
 
-  if (!p_iso) return NULL;
+  if (!p_iso)
+     return NULL;
 
+  p_iso->header.u_type = CDIO_HEADER_TYPE_ISO;
   p_iso->stream = cdio_stdio_new( psz_path );
   if (NULL == p_iso->stream)
     goto error;
@@ -299,8 +304,8 @@ check_pvd (const iso9660_pvd_t *p_pvd, cdio_log_level_t log_level)
 
   if (strncmp (p_pvd->id, ISO_STANDARD_ID, strlen (ISO_STANDARD_ID)))
     {
-      cdio_log (log_level, "unexpected ID encountered (expected `"
-		ISO_STANDARD_ID "', got `%.5s')", p_pvd->id);
+      cdio_log (log_level, "unexpected ID encountered (expected '"
+		ISO_STANDARD_ID "', got '%.5s')", p_pvd->id);
       return false;
     }
   return true;
@@ -774,55 +779,156 @@ iso9660_check_dir_block_end(iso9660_dir_t *p_iso9660_dir, unsigned *offset)
   return false;
 }
 
+static inline bool
+_iso9660_is_rock_ridge_enabled(void* p_image)
+{
+  cdio_header_t* p_header = (cdio_header_t*)p_image;
 
+  if (!p_header)
+    return false;
+  if (p_header->u_type == CDIO_HEADER_TYPE_ISO) {
+    iso9660_t* p_iso = (iso9660_t*)p_image;
+    return (p_iso->iso_extension_mask & ISO_EXTENSION_ROCK_RIDGE);
+  }
+  /* Rock Ridge is always enabled for other types */
+  return true;
+}
+
+/*!
+  Convert a directory record name to a 0-terminated string.
+  One of parameters alloc_result and cpy_result should be non-NULL to take
+  the result.
+*/
+static bool
+_iso9660_recname_to_cstring(const char *src, size_t src_len,
+			    cdio_utf8_t **alloc_result,
+			    cdio_utf8_t *cpy_result, uint8_t u_joliet_level)
+{
+#ifdef HAVE_JOLIET
+  if (u_joliet_level) {
+    int i_inlen = src_len;
+    cdio_utf8_t *p_psz_out = NULL;
+
+    if (cdio_charset_to_utf8(src, i_inlen, &p_psz_out, "UCS-2BE")) {
+      if (cpy_result != NULL)
+        strcpy(cpy_result, p_psz_out);
+      if (alloc_result != NULL)
+        *alloc_result = p_psz_out;
+      else
+        free(p_psz_out);
+    } else {
+      return false;
+    }
+  } else
+#endif /*HAVE_JOLIET*/
+  {
+    if (alloc_result != NULL) {
+      *alloc_result = calloc(1, src_len + 1);
+      if (*alloc_result == NULL)
+	return false;
+      strncpy(*alloc_result, src, src_len);
+      (*alloc_result)[src_len] = 0;
+    }
+    if (cpy_result != NULL) {
+      strncpy(cpy_result, src, src_len);
+      cpy_result[src_len] = 0;
+    }
+  }
+  return true;
+}
 
 static iso9660_stat_t *
-_iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
+_iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir,
+			 iso9660_stat_t *last_p_stat,
+			 void* p_image,
+			 bool_3way_t b_xa,
 			 uint8_t u_joliet_level)
 {
   uint8_t dir_len= iso9660_get_dir_len(p_iso9660_dir);
   iso711_t i_fname;
   unsigned int stat_len;
-  iso9660_stat_t *p_stat = NULL;
-  bool err;
+  iso9660_stat_t *p_stat = last_p_stat;
+  char rr_fname[256] = "";
+  int i_rr_fname = 0;
+  lsn_t extent_lsn;
+  bool first_extent;
 
   if (!dir_len) return NULL;
 
-  i_fname  = from_711(p_iso9660_dir->filename.len);
+  i_fname = from_711(p_iso9660_dir->filename.len);
 
   /* .. string in statbuf is one longer than in p_iso9660_dir's listing '\1' */
-  stat_len      = sizeof(iso9660_stat_t)+i_fname+2;
+  stat_len = sizeof(iso9660_stat_t) + i_fname + 2;
 
-  p_stat          = calloc(1, stat_len);
-  if (!p_stat)
-    {
+  /* Reuse multiextent p_stat if not NULL */
+  if (!p_stat) {
+    p_stat = calloc(1, stat_len);
+    first_extent = true;
+  } else {
+    /* Ignore Rock Ridge Deep Directory RE entries */
+    if (p_stat->rr.u_su_fields & ISO_ROCK_SUF_RE)
+      goto fail;
+    first_extent = false;
+  }
+  if (!p_stat) {
     cdio_warn("Couldn't calloc(1, %d)", stat_len);
     return NULL;
-    }
+  }
   p_stat->type    = (p_iso9660_dir->file_flags & ISO_DIRECTORY)
     ? _STAT_DIR : _STAT_FILE;
-  p_stat->lsn     = from_733_with_err (p_iso9660_dir->extent, &err);
-  if (err) {
-    free(p_stat);
-    return NULL;
-  }
-  p_stat->size    = from_733_with_err (p_iso9660_dir->size, &err);
-  if (err) {
-    free(p_stat);
-    return NULL;
-  }
-  p_stat->secsize = _cdio_len2blocks (p_stat->size, ISO_BLOCKSIZE);
-  p_stat->rr.b3_rock = dunno; /*FIXME should do based on mask */
-  p_stat->b_xa    = false;
 
-  {
-    char rr_fname[256] = "";
+  /* Test for gaps between extents. Important: Use previous .total_size */
+  extent_lsn = from_733 (p_iso9660_dir->extent);
+  if (p_stat->total_size > 0) {
+    /* This is a follow-up extent. Check for a gap. */
+    if (p_stat->lsn + p_stat->total_size / ISO_BLOCKSIZE != extent_lsn
+	|| p_stat->total_size % ISO_BLOCKSIZE) {
+      /* Gap detected. Throw error. */
+      cdio_warn("Non-contiguous data extents with '%s'", p_stat->filename);
+      goto fail;
+    }
+  } else if (first_extent) {
+    p_stat->lsn = extent_lsn;
+  }
+  /* Only now update .total_size */
+  p_stat->total_size += from_733(p_iso9660_dir->size);
 
-    int  i_rr_fname =
 #ifdef HAVE_ROCK
-      get_rock_ridge_filename(p_iso9660_dir, rr_fname, p_stat);
-#else
-      0;
+  if (_iso9660_is_rock_ridge_enabled(p_image))
+    p_stat->rr.b3_rock = dunno;
+#endif
+  p_stat->b_xa = false;
+
+#ifndef DO_NOT_WANT_COMPATIBILITY
+  if (first_extent) {
+    p_stat->size = from_733(p_iso9660_dir->size);
+    p_stat->secsize = CDIO_EXTENT_BLOCKS(p_stat->size);
+  }
+#endif /* DO_NOT_WANT_COMPATIBILITY */
+
+  /* Only resolve the full filename when we're not dealing with extent */
+  if ((p_iso9660_dir->file_flags & ISO_MULTIEXTENT) == 0) {
+    /* Check if this is the last part of a multiextent file */
+    if (!first_extent) {
+      cdio_utf8_t *p_psz_out = NULL;
+      int bad_multi;
+      int i_inlen = i_fname;
+
+      if (!_iso9660_recname_to_cstring(&p_iso9660_dir->filename.str[1],
+				       i_inlen, &p_psz_out, NULL,
+				       u_joliet_level))
+	goto fail;
+      bad_multi = (strcmp(p_stat->filename, p_psz_out) != 0);
+      free(p_psz_out);
+      if (bad_multi) {
+	cdio_warn("Non consecutive multiextent file parts for '%s'",
+		  p_stat->filename);
+	goto fail;
+      }
+    }
+
+#ifdef HAVE_ROCK
+    i_rr_fname = get_rock_ridge_filename(p_iso9660_dir, p_image, rr_fname, p_stat);
 #endif
 
     if (i_rr_fname > 0) {
@@ -830,47 +936,41 @@ _iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
 	/* realloc gives valgrind errors */
 	iso9660_stat_t *p_stat_new =
 	  calloc(1, sizeof(iso9660_stat_t)+i_rr_fname+2);
-        if (!p_stat_new)
-          {
-          cdio_warn("Couldn't calloc(1, %d)", (int)(sizeof(iso9660_stat_t)+i_rr_fname+2));
-	  free(p_stat);
-          return NULL;
-          }
+	if (!p_stat_new) {
+	  cdio_warn("Couldn't calloc(1, %d)", (int)(sizeof(iso9660_stat_t)+i_rr_fname+2));
+	  goto fail;
+	}
 	memcpy(p_stat_new, p_stat, stat_len);
 	free(p_stat);
 	p_stat = p_stat_new;
       }
       strncpy(p_stat->filename, rr_fname, i_rr_fname+1);
     } else {
-      if ('\0' == p_iso9660_dir->filename.str[1] && 1 == i_fname)
-	strncpy (p_stat->filename, ".", sizeof("."));
-      else if ('\1' == p_iso9660_dir->filename.str[1] && 1 == i_fname)
-	strncpy (p_stat->filename, "..", sizeof(".."));
-#ifdef HAVE_JOLIET
-      else if (u_joliet_level) {
+      if ('\0' == p_iso9660_dir->filename.str[1] && 1 == i_fname) {
+	strncpy (p_stat->filename, ".", strlen(".")+1);
+      } else if ('\1' == p_iso9660_dir->filename.str[1] && 1 == i_fname) {
+	strncpy (p_stat->filename, "..", strlen("..")+1);
+      } else {
 	int i_inlen = i_fname;
-	cdio_utf8_t *p_psz_out = NULL;
-	if (cdio_charset_to_utf8(&p_iso9660_dir->filename.str[1], i_inlen,
-                             &p_psz_out, "UCS-2BE")) {
-          strncpy(p_stat->filename, p_psz_out, i_fname);
-          free(p_psz_out);
-        }
-        else {
-          free(p_stat);
-          return NULL;
-        }
-      }
-#endif /*HAVE_JOLIET*/
-      else {
-	strncpy (p_stat->filename, &p_iso9660_dir->filename.str[1], i_fname);
+
+	if (!_iso9660_recname_to_cstring(&p_iso9660_dir->filename.str[1],
+					 i_inlen, NULL, p_stat->filename,
+					 u_joliet_level))
+	  goto fail;
       }
     }
-  }
+  } else {
+    int i_inlen = i_fname;
 
+    if (!_iso9660_recname_to_cstring(&p_iso9660_dir->filename.str[1],
+				     i_inlen, NULL, p_stat->filename,
+				     u_joliet_level))
+      goto fail;
+  }
 
   iso9660_get_dtime(&(p_iso9660_dir->recording_time), true, &(p_stat->tm));
 
-  if (dir_len < sizeof (iso9660_dir_t)) {
+  if (dir_len < sizeof(iso9660_dir_t)) {
     iso9660_stat_free(p_stat);
     return NULL;
   }
@@ -916,6 +1016,9 @@ _iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
   }
   return p_stat;
 
+fail:
+  iso9660_stat_free(p_stat);
+  return NULL;
 }
 
 /*!
@@ -988,8 +1091,8 @@ _fs_stat_root (CdIo_t *p_cdio)
     p_iso9660_dir = &(p_env->pvd.root_directory_record) ;
 #endif
 
-    p_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, b_xa,
-				      p_env->u_joliet_level);
+    p_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, NULL, p_cdio,
+				      b_xa, p_env->u_joliet_level);
     return p_stat;
   }
 
@@ -1009,7 +1112,8 @@ _ifs_stat_root (iso9660_t *p_iso)
   p_iso9660_dir = &(p_iso->pvd.root_directory_record) ;
 #endif
 
-  p_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, p_iso->b_xa,
+  p_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, NULL,
+				    p_iso, p_iso->b_xa,
 				    p_iso->u_joliet_level);
   return p_stat;
 }
@@ -1020,8 +1124,11 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
 {
   unsigned offset = 0;
   uint8_t *_dirbuf = NULL;
+  uint32_t blocks;
   iso9660_stat_t *p_stat;
   generic_img_private_t *p_env = (generic_img_private_t *) p_cdio->env;
+  iso9660_stat_t *p_iso9660_stat = NULL;
+  bool skip_following_extents = false;
 
   if (!splitpath[0])
     {
@@ -1040,29 +1147,42 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
     return NULL;
 
   cdio_assert (_root->type == _STAT_DIR);
+  blocks = CDIO_EXTENT_BLOCKS(_root->total_size);
 
-  _dirbuf = calloc(1, _root->secsize * ISO_BLOCKSIZE);
+  _dirbuf = calloc(1, blocks * ISO_BLOCKSIZE);
   if (!_dirbuf)
     {
-    cdio_warn("Couldn't calloc(1, %d)", _root->secsize * ISO_BLOCKSIZE);
+    cdio_warn("Couldn't calloc(1, %d)", blocks * ISO_BLOCKSIZE);
     return NULL;
     }
 
   if (cdio_read_data_sectors (p_cdio, _dirbuf, _root->lsn, ISO_BLOCKSIZE,
-			      _root->secsize))
+			      blocks))
       return NULL;
 
-  while (offset < (_root->secsize * ISO_BLOCKSIZE))
+  while (offset < (blocks * ISO_BLOCKSIZE))
     {
       iso9660_dir_t *p_iso9660_dir = (void *) &_dirbuf[offset];
-      iso9660_stat_t *p_iso9660_stat;
       int cmp;
 
       if (iso9660_check_dir_block_end(p_iso9660_dir, &offset))
 	continue;
 
-      p_iso9660_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, dunno,
-					p_env->u_joliet_level);
+      if (skip_following_extents) {
+	/* Do not register remaining extents of ill file */
+	p_iso9660_stat = NULL;
+      } else {
+	p_iso9660_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, p_iso9660_stat,
+				(CdIo_t*)p_cdio, dunno, p_env->u_joliet_level);
+	if (NULL == p_iso9660_stat)
+	  skip_following_extents = true; /* Start ill file mode */
+      }
+      if ((p_iso9660_dir->file_flags & ISO_MULTIEXTENT) == 0)
+	skip_following_extents = false; /* Ill or not: The file ends now */
+
+      if (NULL == p_iso9660_stat ||
+	  (p_iso9660_dir->file_flags & ISO_MULTIEXTENT))
+	goto skip_to_next_record;
 
       cmp = strcmp(splitpath[0], p_iso9660_stat->filename);
 
@@ -1076,7 +1196,7 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
 	  if (!trans_fname) {
 	    cdio_warn("can't allocate %lu bytes",
 		      (long unsigned int) strlen(p_iso9660_stat->filename));
-	    free(p_iso9660_stat);
+	    iso9660_stat_free(p_iso9660_stat);
 	    return NULL;
 	  }
 	  iso9660_name_translate_ext(p_iso9660_stat->filename, trans_fname,
@@ -1095,11 +1215,14 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
       }
 
       iso9660_stat_free(p_iso9660_stat);
+      p_iso9660_stat = NULL;
+
+skip_to_next_record:;
 
       offset += iso9660_get_dir_len(p_iso9660_dir);
     }
 
-  cdio_assert (offset == (_root->secsize * ISO_BLOCKSIZE));
+  cdio_assert (offset == (blocks * ISO_BLOCKSIZE));
 
   /* not found */
   free (_dirbuf);
@@ -1112,11 +1235,13 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 {
   unsigned offset = 0;
   uint8_t *_dirbuf = NULL;
-  int ret;
+  uint32_t blocks;
+  int ret, cmp;
+  iso9660_stat_t *p_stat = NULL;
+  iso9660_dir_t *p_iso9660_dir = NULL;
 
   if (!splitpath[0])
     {
-      iso9660_stat_t *p_stat;
       unsigned int len=sizeof(iso9660_stat_t) + strlen(_root->filename)+1;
       p_stat = calloc(1, len);
       cdio_assert (p_stat != NULL);
@@ -1133,30 +1258,30 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 
   cdio_assert (_root->type == _STAT_DIR);
 
-  _dirbuf = calloc(1, _root->secsize * ISO_BLOCKSIZE);
+  blocks = CDIO_EXTENT_BLOCKS(_root->total_size);
+  _dirbuf = calloc(1, blocks * ISO_BLOCKSIZE);
   if (!_dirbuf)
     {
-    cdio_warn("Couldn't calloc(1, %d)", _root->secsize * ISO_BLOCKSIZE);
+    cdio_warn("Couldn't calloc(1, %d)", blocks * ISO_BLOCKSIZE);
     return NULL;
     }
 
-  ret = iso9660_iso_seek_read (p_iso, _dirbuf, _root->lsn, _root->secsize);
-  if (ret!=ISO_BLOCKSIZE*_root->secsize) {
+  ret = iso9660_iso_seek_read (p_iso, _dirbuf, _root->lsn, blocks);
+  if (ret != blocks * ISO_BLOCKSIZE) {
     free(_dirbuf);
     return NULL;
   }
 
-  while (offset < (_root->secsize * ISO_BLOCKSIZE))
+  for (offset = 0; offset < (blocks * ISO_BLOCKSIZE);
+       offset += iso9660_get_dir_len(p_iso9660_dir))
     {
-      iso9660_dir_t *p_iso9660_dir = (void *) &_dirbuf[offset];
-      iso9660_stat_t *p_stat;
-      int cmp;
+      p_iso9660_dir = (void *) &_dirbuf[offset];
 
       if (iso9660_check_dir_block_end(p_iso9660_dir, &offset))
 	continue;
 
-      p_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, p_iso->b_xa,
-					p_iso->u_joliet_level);
+      p_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, p_stat, p_iso,
+					p_iso->b_xa, p_iso->u_joliet_level);
 
       if (!p_stat) {
 	cdio_warn("Bad directory information for %s", splitpath[0]);
@@ -1164,6 +1289,10 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 	return NULL;
       }
 
+      /* If we have multiextent file parts, loop until the last one */
+      if (p_iso9660_dir->file_flags & ISO_MULTIEXTENT)
+        continue;
+
       cmp = strcmp(splitpath[0], p_stat->filename);
 
       if ( 0 != cmp && 0 == p_iso->u_joliet_level
@@ -1176,7 +1305,7 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 	  if (!trans_fname) {
 	    cdio_warn("can't allocate %lu bytes",
 		      (long unsigned int) strlen(p_stat->filename));
-	    free(p_stat);
+	    iso9660_stat_free(p_stat);
 	    return NULL;
 	  }
 	  iso9660_name_translate_ext(p_stat->filename, trans_fname,
@@ -1194,11 +1323,10 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 	return ret_stat;
       }
       iso9660_stat_free(p_stat);
-
-      offset += iso9660_get_dir_len(p_iso9660_dir);
+      p_stat = NULL;
     }
 
-  cdio_assert (offset == (_root->secsize * ISO_BLOCKSIZE));
+  cdio_assert (offset == (blocks * ISO_BLOCKSIZE));
 
   /* not found */
   free (_dirbuf);
@@ -1239,7 +1367,7 @@ iso9660_fs_stat (CdIo_t *p_cdio, const char psz_path[])
 
   p_psz_splitpath = _cdio_strsplit (psz_path, '/');
   p_stat = _fs_stat_traverse (p_cdio, p_root, p_psz_splitpath);
-  free(p_root);
+  iso9660_stat_free(p_root);
   _cdio_strfreev (p_psz_splitpath);
 
   return p_stat;
@@ -1272,7 +1400,7 @@ fs_stat_translate (void *p_image, stat_root_t stat_root,
 
   p_psz_splitpath = _cdio_strsplit (psz_path, '/');
   p_stat = stat_traverse (p_image, p_root, p_psz_splitpath);
-  free(p_root);
+  iso9660_stat_free(p_root);
   _cdio_strfreev (p_psz_splitpath);
 
   return p_stat;
@@ -1342,7 +1470,7 @@ iso9660_ifs_stat (iso9660_t *p_iso, const char psz_path[])
 
   splitpath = _cdio_strsplit (psz_path, '/');
   stat = _fs_iso_stat_traverse (p_iso, p_root, splitpath);
-  free(p_root);
+  iso9660_stat_free(p_root);
   _cdio_strfreev (splitpath);
 
   return stat;
@@ -1363,8 +1491,16 @@ CdioISO9660FileList_t *
 iso9660_fs_readdir (CdIo_t *p_cdio, const char psz_path[])
 {
   generic_img_private_t *p_env;
+  iso9660_dir_t *p_iso9660_dir;
+  iso9660_stat_t *p_iso9660_stat = NULL;
   iso9660_stat_t *p_stat;
 
+  unsigned offset = 0;
+  uint8_t *_dirbuf = NULL;
+  uint32_t blocks;
+  CdioISO9660DirList_t *retval;
+  bool skip_following_extents = false;
+
   if (!p_cdio)   return NULL;
   if (!psz_path) return NULL;
 
@@ -1378,48 +1514,76 @@ iso9660_fs_readdir (CdIo_t *p_cdio, const char psz_path[])
     return NULL;
   }
 
-  {
-    unsigned offset = 0;
-    uint8_t *_dirbuf = NULL;
-    CdioISO9660DirList_t *retval = _cdio_list_new ();
+  /* Check for overflow on 32-bit systems.
+     uint32_t has a limited maximum value, and if p_stat->total_size (the total
+     size of the directory) is very large, the calculation might exceed this limit.
+  */
+  if (p_stat->total_size > SIZE_MAX / ISO_BLOCKSIZE) {
+    cdio_warn("Total size is too large");
+    iso9660_stat_free(p_stat);
+    return NULL;
+  }
 
-    _dirbuf = calloc(1, p_stat->secsize * ISO_BLOCKSIZE);
-    if (!_dirbuf)
-      {
-      cdio_warn("Couldn't calloc(1, %d)", p_stat->secsize * ISO_BLOCKSIZE);
-      iso9660_stat_free(p_stat);
-      iso9660_dirlist_free(retval);
-      return NULL;
-      }
+  blocks = CDIO_EXTENT_BLOCKS(p_stat->total_size);
+  retval = _cdio_list_new ();
 
-    if (cdio_read_data_sectors (p_cdio, _dirbuf, p_stat->lsn,
-				ISO_BLOCKSIZE, p_stat->secsize)) {
+  /* Check for potential integer overflow when calculating total blocks */
+  if (blocks > (SIZE_MAX / ISO_BLOCKSIZE)) {
+    cdio_warn("Total size is too large");
+    iso9660_stat_free(p_stat);
+    return NULL;
+  }
+
+  _dirbuf = calloc(1, blocks * ISO_BLOCKSIZE);
+  if (!_dirbuf)
+    {
+      cdio_warn("Couldn't calloc(1, %d)", blocks * ISO_BLOCKSIZE);
       iso9660_stat_free(p_stat);
       iso9660_dirlist_free(retval);
       return NULL;
     }
 
-    while (offset < (p_stat->secsize * ISO_BLOCKSIZE))
-      {
-	iso9660_dir_t *p_iso9660_dir = (void *) &_dirbuf[offset];
-	iso9660_stat_t *p_iso9660_stat;
+  if (cdio_read_data_sectors (p_cdio, _dirbuf, p_stat->lsn,
+			      ISO_BLOCKSIZE, blocks)) {
+    iso9660_stat_free(p_stat);
+    iso9660_dirlist_free(retval);
+    return NULL;
+  }
 
-	if (iso9660_check_dir_block_end(p_iso9660_dir, &offset))
-  	  continue;
+  while (offset < (blocks * ISO_BLOCKSIZE))
+    {
+      p_iso9660_dir = (void *) &_dirbuf[offset];
 
-	p_iso9660_stat = _iso9660_dir_to_statbuf(p_iso9660_dir, dunno,
-						 p_env->u_joliet_level);
+      if (iso9660_check_dir_block_end(p_iso9660_dir, &offset))
+	continue;
+
+      if (skip_following_extents) {
+	/* Do not register remaining extents of ill file */
+	p_iso9660_stat = NULL;
+      } else {
+	p_iso9660_stat = _iso9660_dir_to_statbuf(p_iso9660_dir,
+						 p_iso9660_stat, p_cdio,
+						 dunno, p_env->u_joliet_level);
+	if (NULL == p_iso9660_stat)
+	  skip_following_extents = true; /* Start ill file mode */
+      }
+      if ((p_iso9660_dir->file_flags & ISO_MULTIEXTENT) == 0)
+	skip_following_extents = false; /* Ill or not: The file ends now */
+
+      if ((p_iso9660_stat) &&
+	  ((p_iso9660_dir->file_flags & ISO_MULTIEXTENT) == 0)) {
 	_cdio_list_append (retval, p_iso9660_stat);
-
-	offset += iso9660_get_dir_len(p_iso9660_dir);
+	p_iso9660_stat = NULL;
       }
 
-    cdio_assert (offset == (p_stat->secsize * ISO_BLOCKSIZE));
+      offset += iso9660_get_dir_len(p_iso9660_dir);
+    }
 
-    free(_dirbuf);
-    iso9660_stat_free(p_stat);
-    return retval;
-  }
+  cdio_assert (offset == (blocks * ISO_BLOCKSIZE));
+
+  free(_dirbuf);
+  iso9660_stat_free(p_stat);
+  return retval;
 }
 
 /*!
@@ -1429,8 +1593,18 @@ iso9660_fs_readdir (CdIo_t *p_cdio, const char psz_path[])
 CdioISO9660FileList_t *
 iso9660_ifs_readdir (iso9660_t *p_iso, const char psz_path[])
 {
+  iso9660_dir_t *p_iso9660_dir;
+  iso9660_stat_t *p_iso9660_stat = NULL;
   iso9660_stat_t *p_stat;
 
+  long int ret;
+  unsigned offset = 0;
+  uint8_t *_dirbuf = NULL;
+  uint32_t blocks;
+  CdioList_t *retval;
+  size_t dirbuf_len;
+  bool skip_following_extents = false;
+
   if (!p_iso)    return NULL;
   if (!psz_path) return NULL;
 
@@ -1442,70 +1616,87 @@ iso9660_ifs_readdir (iso9660_t *p_iso, const char psz_path[])
     return NULL;
   }
 
-  {
-    long int ret;
-    unsigned offset = 0;
-    uint8_t *_dirbuf = NULL;
-    CdioList_t *retval = _cdio_list_new ();
-    const size_t dirbuf_len = p_stat->secsize * ISO_BLOCKSIZE;
+  /* Check for overflow on 32-bit systems.
+     uint32_t has a limited maximum value, and if p_stat->total_size (the total
+     size of the directory) is very large, the calculation might exceed this limit.
+  */
 
-
-    if (!dirbuf_len)
-      {
-        cdio_warn("Invalid directory buffer sector size %u", p_stat->secsize);
-	iso9660_stat_free(p_stat);
-	_cdio_list_free (retval, true, NULL);
-        return NULL;
-      }
-
-    _dirbuf = calloc(1, dirbuf_len);
-    if (!_dirbuf)
-      {
-        cdio_warn("Couldn't calloc(1, %lu)", (unsigned long)dirbuf_len);
-	iso9660_stat_free(p_stat);
-	_cdio_list_free (retval, true, NULL);
-        return NULL;
-      }
-
-    ret = iso9660_iso_seek_read (p_iso, _dirbuf, p_stat->lsn, p_stat->secsize);
-    if (ret != dirbuf_len) 	  {
-      _cdio_list_free (retval, true, NULL);
-      iso9660_stat_free(p_stat);
-      free (_dirbuf);
-      return NULL;
-    }
-
-    while (offset < (dirbuf_len))
-      {
-	iso9660_dir_t *p_iso9660_dir = (void *) &_dirbuf[offset];
-	iso9660_stat_t *p_iso9660_stat;
-
-	if (iso9660_check_dir_block_end(p_iso9660_dir, &offset))
-	  continue;
-
-	p_iso9660_stat = _iso9660_dir_to_statbuf(p_iso9660_dir, p_iso->b_xa,
-						 p_iso->u_joliet_level);
-
-	if (p_iso9660_stat)
-	  _cdio_list_append (retval, p_iso9660_stat);
-	else {
-	  cdio_warn("Invalid directory stat at offset %lu", (unsigned long)offset);
-	  break;
-	}
-
-	offset += iso9660_get_dir_len(p_iso9660_dir);
-      }
-
-    free (_dirbuf);
+  if (p_stat->total_size > SIZE_MAX / ISO_BLOCKSIZE) {
+    cdio_warn("Total size is too large");
     iso9660_stat_free(p_stat);
+    return NULL;
+  }
 
-    if (offset != dirbuf_len) {
-      _cdio_list_free (retval, true, (CdioDataFree_t) iso9660_stat_free);
+  blocks = CDIO_EXTENT_BLOCKS(p_stat->total_size);
+  dirbuf_len = blocks * ISO_BLOCKSIZE;
+  retval = _cdio_list_new ();
+
+  if (!dirbuf_len)
+    {
+      cdio_warn("Invalid directory buffer sector size %u", blocks);
+      iso9660_stat_free(p_stat);
+      _cdio_list_free (retval, true, NULL);
       return NULL;
     }
 
-    return retval;
+  _dirbuf = calloc(1, dirbuf_len);
+  if (!_dirbuf)
+    {
+      cdio_warn("Couldn't calloc(1, %lu)", (unsigned long)dirbuf_len);
+      iso9660_stat_free(p_stat);
+      _cdio_list_free (retval, true, NULL);
+      return NULL;
+    }
+
+  ret = iso9660_iso_seek_read (p_iso, _dirbuf, p_stat->lsn, blocks);
+  if (ret != dirbuf_len) 	  {
+    _cdio_list_free (retval, true, NULL);
+    iso9660_stat_free(p_stat);
+    free (_dirbuf);
+    return NULL;
   }
+
+  while (offset < (dirbuf_len))
+    {
+      p_iso9660_dir = (void *) &_dirbuf[offset];
+
+      if (iso9660_check_dir_block_end(p_iso9660_dir, &offset))
+	continue;
+
+      if (skip_following_extents) {
+	/* Do not register remaining extents of ill file */
+	p_iso9660_stat = NULL;
+      } else {
+	p_iso9660_stat = _iso9660_dir_to_statbuf(p_iso9660_dir,
+						 p_iso9660_stat,
+						 p_iso,
+						 p_iso->b_xa,
+						 p_iso->u_joliet_level);
+	if (NULL == p_iso9660_stat)
+	  skip_following_extents = true; /* Start ill file mode */
+	else if (p_iso9660_stat->rr.u_su_fields & ISO_ROCK_SUF_RE)
+	  continue; /* Ignore RE entries */
+      }
+      if ((p_iso9660_dir->file_flags & ISO_MULTIEXTENT) == 0)
+	skip_following_extents = false; /* Ill or not: The file ends now */
+      if ((p_iso9660_stat) &&
+	  ((p_iso9660_dir->file_flags & ISO_MULTIEXTENT) == 0)) {
+	_cdio_list_append(retval, p_iso9660_stat);
+	p_iso9660_stat = NULL;
+      }
+
+      offset += iso9660_get_dir_len(p_iso9660_dir);
+    }
+
+  free (_dirbuf);
+  iso9660_stat_free(p_stat);
+
+  if (offset != dirbuf_len) {
+    _cdio_list_free (retval, true, (CdioDataFree_t) iso9660_stat_free);
+    return NULL;
+  }
+
+  return retval;
 }
 
 typedef CdioISO9660FileList_t * (iso9660_readdir_t)
@@ -1552,7 +1743,8 @@ find_lsn_recurse (void *p_image, iso9660_readdir_t iso9660_readdir,
       }
 
       if (statbuf->lsn == lsn) {
-	const unsigned int len2 = sizeof(iso9660_stat_t)+strlen(statbuf->filename)+1;
+	const unsigned int len2 = sizeof(iso9660_stat_t) +
+				  strlen(statbuf->filename) + 1;
 	iso9660_stat_t *ret_stat = calloc(1, len2);
 	if (!ret_stat)
 	  {
@@ -1563,9 +1755,9 @@ find_lsn_recurse (void *p_image, iso9660_readdir_t iso9660_readdir,
 	    return NULL;
 	  }
 	memcpy(ret_stat, statbuf, len2);
-        iso9660_filelist_free (entlist);
+	iso9660_filelist_free (entlist);
 	iso9660_dirlist_free(dirlist);
-        return ret_stat;
+	return ret_stat;
       }
 
     }
@@ -1585,7 +1777,7 @@ find_lsn_recurse (void *p_image, iso9660_readdir_t iso9660_readdir,
 				   ppsz_full_filename);
 
       if (NULL != ret_stat) {
-	iso9660_dirlist_free(dirlist);
+        iso9660_dirlist_free(dirlist);
         return ret_stat;
       }
     }
@@ -1615,6 +1807,12 @@ iso9660_fs_find_lsn(CdIo_t *p_cdio, lsn_t i_lsn)
     free(psz_full_filename);
   return p_statbuf;
 }
+iso9660_stat_t *
+#if defined(__GNUC__) && !defined(__DARWIN_C_ANSI)
+iso9660_find_fs_lsn(CdIo_t *p_cdio, lsn_t i_lsn) __attribute__ ((alias ("iso9660_fs_find_lsn")));
+#else
+iso9660_find_fs_lsn(CdIo_t *p_cdio, lsn_t i_lsn);
+#endif
 
 /*!
    Given a directory pointer, find the filesystem entry that contains
@@ -1661,6 +1859,55 @@ iso9660_ifs_find_lsn(iso9660_t *p_iso, lsn_t i_lsn)
   return ret;
 }
 
+#ifdef HAVE_ROCK
+/* Some compilers complain if the prototype is not defined */
+iso9660_stat_t *
+_iso9660_dd_find_lsn(void* p_image, lsn_t i_lsn);
+
+/* Same as above for Rock Ridge deep directory traversing. */
+iso9660_stat_t *
+_iso9660_dd_find_lsn(void* p_image, lsn_t i_lsn)
+{
+  cdio_header_t* p_header = (cdio_header_t*)p_image;
+  char* psz_full_filename = NULL;
+  void* p_image_dd;
+  iso9660_readdir_t* f_readdir;
+  iso9660_stat_t* ret;
+  size_t size;
+
+  switch(p_header->u_type) {
+  case CDIO_HEADER_TYPE_ISO:
+    size = sizeof(iso9660_t);
+    f_readdir = (iso9660_readdir_t*)iso9660_ifs_readdir;
+    break;
+  case CDIO_HEADER_TYPE_CDIO:
+    size = sizeof(CdIo_t);
+    f_readdir = (iso9660_readdir_t*)iso9660_fs_readdir;
+    break;
+  default:
+    cdio_assert(false);
+    return NULL;
+  }
+
+  /* Work with a duplicate to allow concurrency. */
+  p_image_dd = calloc(1, size);
+  if (!p_image_dd) {
+    cdio_warn("Memory duplication error");
+    return NULL;
+  }
+  memcpy(p_image_dd, p_image, size);
+
+  /* Disable the deep directory flag so we can process all entries */
+  p_header = (cdio_header_t*)p_image_dd;
+  p_header->u_flags |= CDIO_HEADER_FLAGS_DISABLE_RR_DD;
+  ret = find_lsn_recurse(p_image_dd, f_readdir, "/", i_lsn, &psz_full_filename);
+  if (psz_full_filename != NULL)
+    free(psz_full_filename);
+  free(p_image_dd);
+  return ret;
+}
+#endif /* HAVE ROCK */
+
 /*!
    Given a directory pointer, find the filesystem entry that contains
    lsn and return information about it.
@@ -1735,6 +1982,7 @@ iso_have_rr_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 {
   unsigned offset = 0;
   uint8_t *_dirbuf = NULL;
+  uint32_t blocks;
   int ret;
   bool_3way_t have_rr = nope;
 
@@ -1745,20 +1993,21 @@ iso_have_rr_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 
   cdio_assert (_root->type == _STAT_DIR);
 
-  _dirbuf = calloc(1, _root->secsize * ISO_BLOCKSIZE);
+   blocks = CDIO_EXTENT_BLOCKS(_root->total_size);
+  _dirbuf = calloc(1, blocks * ISO_BLOCKSIZE);
   if (!_dirbuf)
     {
-    cdio_warn("Couldn't calloc(1, %d)", _root->secsize * ISO_BLOCKSIZE);
+    cdio_warn("Couldn't calloc(1, %d)", blocks * ISO_BLOCKSIZE);
     return dunno;
     }
 
-  ret = iso9660_iso_seek_read (p_iso, _dirbuf, _root->lsn, _root->secsize);
-  if (ret!=ISO_BLOCKSIZE*_root->secsize) {
+  ret = iso9660_iso_seek_read (p_iso, _dirbuf, _root->lsn, blocks);
+  if (ret != blocks * ISO_BLOCKSIZE) {
     free(_dirbuf);
     return false;
   }
 
-  while (offset < (_root->secsize * ISO_BLOCKSIZE))
+  while (offset < (blocks * ISO_BLOCKSIZE))
     {
       iso9660_dir_t *p_iso9660_dir = (void *) &_dirbuf[offset];
       iso9660_stat_t *p_stat;
@@ -1767,8 +2016,8 @@ iso_have_rr_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
       if (iso9660_check_dir_block_end(p_iso9660_dir, &offset))
 	continue;
 
-      p_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, p_iso->b_xa,
-					p_iso->u_joliet_level);
+      p_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, NULL, p_iso,
+					p_iso->b_xa, p_iso->u_joliet_level);
       have_rr = p_stat->rr.b3_rock;
       if ( have_rr != yep) {
 	if (strlen(splitpath[0]) == 0)
@@ -1777,7 +2026,7 @@ iso_have_rr_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 	  have_rr = iso_have_rr_traverse (p_iso, p_stat, &splitpath[i_last_component],
 					  pu_file_limit);
       }
-      free(p_stat);
+      iso9660_stat_free(p_stat);
       if (have_rr != nope) {
 	free (_dirbuf);
 	return have_rr;
@@ -1791,7 +2040,7 @@ iso_have_rr_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
       }
     }
 
-  cdio_assert (offset == (_root->secsize * ISO_BLOCKSIZE));
+  cdio_assert (offset == (blocks * ISO_BLOCKSIZE));
 
   /* not found */
   free (_dirbuf);
@@ -1827,7 +2076,7 @@ iso9660_have_rr(iso9660_t *p_iso, uint64_t u_file_limit)
   if (u_file_limit == 0) u_file_limit = UINT64_MAX;
 
   is_rr = iso_have_rr_traverse (p_iso, p_root, p_psz_splitpath, &u_file_limit);
-  free(p_root);
+  iso9660_stat_free(p_root);
   free(p_psz_splitpath[0]);
   free(p_psz_splitpath[1]);
 
