@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003, 2004, 2005, 2008, 2011 
+  Copyright (C) 2003, 2004, 2005, 2008, 2011, 2012, 2024
   Rocky Bernstein <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
@@ -25,11 +25,15 @@
 # define __CDIO_CONFIG_H__ 1
 #endif
 
-#include <stdio.h>
-#include <sys/types.h>
 #include <cdio/cdio.h>
 #include <cdio/logging.h>
 #include <cdio/iso9660.h>
+#ifdef HAVE_STDIO_H
+#include <stdio.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -56,7 +60,7 @@
 #endif
 
 #ifdef ENABLE_NLS
-#include <locale.h>
+#    include <locale.h>
 #    include <libintl.h>
 #    define _(String) dgettext ("cdinfo", String)
 #else
@@ -82,17 +86,17 @@
 
 #define DEBUG 1
 #if DEBUG
-#define dbg_print(level, s, args...) \
+#define dbg_print(level, s, ...) \
    if (opts.debug_level >= level) \
-     report(stderr, "%s: "s, __func__ , ##args)
+     report(stderr, "%s: "s, __func__ , __VA_ARGS__)
 #else
 #define dbg_print(level, s, args...) 
 #endif
 
-#define err_exit(fmt, args...) \
-  report(stderr, "%s: "fmt, program_name, ##args); \
-  myexit(p_cdio, EXIT_FAILURE)		     
-  
+#define err_exit(fmt, ...) \
+  report(stderr, "%s: "fmt, program_name, __VA_ARGS__); \
+  myexit(p_cdio, EXIT_FAILURE)
+
 typedef enum
 {
   INPUT_AUTO,
