@@ -1,31 +1,34 @@
 /*
-    $Id: check_sizeof.c,v 1.4 2005/02/12 10:23:18 rocky Exp $
+    Copyright (C) 2001, 2008, 2011 Herbert Valerio Riedel <hvr@gnu.org>
 
-    Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
+     This program is free software: you can redistribute it and/or modify
+     it under the terms of the GNU General Public License as published by
+     the Free Software Foundation, either version 3 of the License, or
+     (at your option) any later version.
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+     This program is distributed in the hope that it will be useful,
+     but WITHOUT ANY WARRANTY; without even the implied warranty of
+     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+     GNU General Public License for more details.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+     You should have received a copy of the GNU General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
-
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+#include "config.h"
+#define __CDIO_CONFIG_H__ 1
 #endif
 
 #include <assert.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
+#endif
 #include <math.h>
 
 #include <cdio/iso9660.h>
@@ -35,26 +38,26 @@
 #include "iso9660_private.h"
 
 #define CHECK_SIZEOF(typnam) { \
-  printf ("checking sizeof (%s) ...", #typnam); \
+  printf ("-- checking sizeof (%s) ...", #typnam); \
   if (sizeof (typnam) != (typnam##_SIZEOF)) { \
       printf ("failed!\n==> sizeof (%s) == %d (but should be %d)\n", \
               #typnam, (int)sizeof(typnam), (int)(typnam##_SIZEOF)); \
       fail++; \
-  } else { pass++; printf ("ok!\n"); } \
+  } else printf ("ok!\n"); \
 }
 
 #define CHECK_SIZEOF_STRUCT(typnam) { \
-  printf ("checking sizeof (struct %s) ...", #typnam); \
+  printf ("-- checking sizeof (struct %s) ...", #typnam); \
   if (sizeof (struct typnam) != (struct_##typnam##_SIZEOF)) { \
       printf ("failed!\n==> sizeof (struct %s) == %d (but should be %d)\n", \
               #typnam, (int)sizeof(struct typnam), (int)(struct_##typnam##_SIZEOF)); \
       fail++; \
-  } else { pass++; printf ("ok!\n"); } \
+  } else printf ("ok!\n"); \
 }
 
 int main (int argc, const char *argv[])
 {
-  unsigned fail = 0, pass = 0;
+  unsigned fail = 0;
 
   /* <cdio/types.h> */
   CHECK_SIZEOF(msf_t);
